﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2011/01/27 初版
//
// FVC05 4ch only Sample Program
// -----------------------------------------------------------------
namespace FVC05_4chSample
{
    public partial class Form_FVC05_4CHSAMPLE : Form
    {
        //Video1CH0用画像メモリ格納領域
        private FVIL.Data.CFviImage m_image1_CH0 = new FVIL.Data.CFviImage();
        //Video1CH1用画像メモリ格納領域
        private FVIL.Data.CFviImage m_image1_CH1 = new FVIL.Data.CFviImage();
        //Video2CH0用画像メモリ格納領域
        private FVIL.Data.CFviImage m_image2_CH0 = new FVIL.Data.CFviImage();
        //Video2CH1用画像メモリ格納領域
        private FVIL.Data.CFviImage m_image2_CH1 = new FVIL.Data.CFviImage();
        //FVC05クラス
        private FVIL.Video.CFviVideoFVC05 m_video1 = new FVIL.Video.CFviVideoFVC05();
        private FVIL.Video.CFviVideoFVC05 m_video2 = new FVIL.Video.CFviVideoFVC05();
        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();

        //iniファイル名（フルパス）
        public string m_strIniFilePath;
        //ビデオデバイスのオープン状態
        private bool m_flgVideo1Opened;
        private bool m_flgVideo2Opened;
        //フリーランの状態
        private bool m_flgFreerun;

        //入力画像タイプ
        enum GrabImagType
        {
            GRAB_GRAY,
            GRAB_BIN        }
        //入力画像の状態
        private GrabImagType m_imgtype = GrabImagType.GRAB_GRAY;

        public Form_FVC05_4CHSAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            //使用許諾の確認
            dialogLicence licence_dialog = new dialogLicence();
            if (licence_dialog.ShowDialog(this) != DialogResult.OK)
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

             //iniファイル選択
            fnSetIniFile();

            //描画imageのセット
            CFviImageViewVideo1CH0.Image = m_image1_CH0;
            CFviImageViewVideo1CH1.Image = m_image1_CH1;
            CFviImageViewVideo2CH0.Image = m_image2_CH0;
            CFviImageViewVideo2CH1.Image = m_image2_CH1;
        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。

            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_flgVideo1Opened = false;
            m_flgVideo2Opened = false;
            m_flgFreerun = false;

            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0";

            //トリガモードコンボボックス初期化
            ComboBoxVideo1Trigger.Items.Clear();
            ComboBoxVideo1Trigger.Items.Add("ノーマル");
            ComboBoxVideo1Trigger.Items.Add("内部トリガ");
            ComboBoxVideo1Trigger.Items.Add("外部トリガ");
            ComboBoxVideo1Trigger.SelectedIndex = 0;
            ComboBoxVideo2Trigger.Items.Clear();
            ComboBoxVideo2Trigger.Items.Add("ノーマル");
            ComboBoxVideo2Trigger.Items.Add("内部トリガ");
            ComboBoxVideo2Trigger.Items.Add("外部トリガ");
            ComboBoxVideo2Trigger.SelectedIndex = 0;

            //２値化コンボボックス初期化
            ComboBoxBinarize.Items.Clear();
            ComboBoxBinarize.Items.Add("グレイ");
            ComboBoxBinarize.Items.Add("二値");
            ComboBoxBinarize.SelectedIndex = 0;
        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            //iniファイル設定画面
            Form_BROWSEIN formbroseini = new Form_BROWSEIN();

            //iniファイルの選択
            m_strIniFilePath = "";
            if (formbroseini.ShowDialog(this) == DialogResult.OK)
            {
                m_strIniFilePath = formbroseini.strIniFilePath;
            }

            if (m_strIniFilePath.Length > 0)
            {
                try
                {
                    //ビデオデバイスがオープンされていた場合はクローズする
                    if (m_video1.IsOpen())
                    {
                        m_video1.Close();
                        m_flgVideo1Opened = false;
                    }
                    if (m_video2.IsOpen())
                    {
                        m_video2.Close();
                        m_flgVideo2Opened = false;
                    }

                    //デバイスオープン
                    m_video1.Open(-1, 0);
                    m_flgVideo1Opened = true;
                    m_video2.Open(-1, 0);
                    m_flgVideo2Opened = true;

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);

                    //２カメラ同時取り込みに設定
                    m_video1.SimGrab = 2;
                    m_video2.SimGrab = 2;

                    //iniファイルをコンポーネントに設定
                    if ((m_strIniFilePath.Length > 0))
                    {
                        m_video1.LoadIniFile(m_strIniFilePath);
                        m_video2.LoadIniFile(m_strIniFilePath);
                    }

                    //取込画像を正規化
                    m_video1.ChangeImageSize(m_image1_CH0);
                    m_video1.ChangeImageSize(m_image1_CH1);
                    m_video2.ChangeImageSize(m_image2_CH0);
                    m_video2.ChangeImageSize(m_image2_CH1);

                    //ステータス表示の更新
                    //トリガモードの取得
                    ComboBoxVideo1Trigger.SelectedIndex = (int)m_video1.Trigger;
                    ComboBoxVideo2Trigger.SelectedIndex = (int)m_video2.Trigger;
                    //カメラ露光時間の取得
                    NumericUpDownVideo1Exposure.Value = m_video1.ExposureTime;
                    NumericUpDownVideo2Exposure.Value = m_video2.ExposureTime;

                    //各チャネルのステータスを取得
                    //ビデオ入力オフセットの取得
                    NumericUpDownVideo1CH0Offset.Value = m_video1.VideoOffsetCH0;
                    NumericUpDownVideo1CH1Offset.Value = m_video1.VideoOffsetCH1;
                    NumericUpDownVideo2CH0Offset.Value = m_video2.VideoOffsetCH0;
                    NumericUpDownVideo2CH1Offset.Value = m_video2.VideoOffsetCH1;
                    //２値化閾値の取得
                    NumericUpDownVideo1CH0Threshold.Value = m_video1.BinarizeCH0.Threshold;
                    NumericUpDownVideo1CH1Threshold.Value = m_video1.BinarizeCH1.Threshold;
                    NumericUpDownVideo2CH0Threshold.Value = m_video2.BinarizeCH0.Threshold;
                    NumericUpDownVideo2CH1Threshold.Value = m_video2.BinarizeCH1.Threshold;

                    //２値化設定(CH0)の取得
                    ComboBoxBinarize.SelectedIndex = (int)m_video1.BinarizeCH0.Mode;

                    //同時取り込みの設定
                    m_video1.SimGrab = 2;               //２カメラ同時取り込み
                    m_video2.SimGrab = 2;               //２カメラ同時取り込み
                    //トリガ共有モードの設定
                    m_video1.TrgSharingMode = 0;        //独立モード
                    m_video2.TrgSharingMode = 0;        //独立モード
                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {
            double msec = 0;
            int Status1 = 1;
            int Status2 = 1;

            try
            {
                if ((m_flgVideo1Opened == true) && (m_flgVideo2Opened == true))
                {
                    //時間測定開始
                    m_timer.Start();
                    if (CheckBoxSimulCapture.Checked == true)
                    {
                        //カメラ取り込み
                        m_video2.GrabImageASync(m_image2_CH0, m_image2_CH1);
                        m_video1.GrabImageSync(m_image1_CH0, m_image1_CH1);
                    }
                    else
                    {
                        //カメラ取り込み
                        m_video1.GrabImageASync(m_image1_CH0, m_image1_CH1);
                        m_video2.GrabImageASync(m_image2_CH0, m_image2_CH1);
                    }
                    while ((Status1 != 0) || (Status2 != 0))
                    {
                        Status1 = m_video1.GrabStatus;
                        Status2 = m_video2.GrabStatus;
                    }
                    //時間測定終了
                    msec = m_timer.Stop();
                    //測定時間表示
                    TextBoxTime.Text = msec.ToString("0.000");
                    //再描画
                    CFviImageViewVideo1CH0.Refresh();
                    CFviImageViewVideo1CH1.Refresh();
                    CFviImageViewVideo2CH0.Refresh();
                    CFviImageViewVideo2CH1.Refresh();
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Grab Continueボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// フリーラン開始・停止。
        /// </remarks>
        private void ButtonGrabContinue_Click(object sender, EventArgs e)
        {
            if (m_flgFreerun == true)
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
            }
            else
            {
                ButtonGrabContinue.Text = "Stop";
                ButtonGrabOne.Enabled = false;
                m_flgFreerun = true;
                timer1.Interval = 50;
                timer1.Start();
            }
            Application.DoEvents();

        }

        /// <summary>
        /// フリーラン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 連続カメラ取り込み。
        /// </remarks>
        private void timer1_Tick(object sender, EventArgs e)
        {
            double msec = 0;
            int Status1 = 1;
            int Status2 = 1;

            timer1.Stop();

            try
            {
                if ((m_flgVideo1Opened == true) && (m_flgVideo2Opened == true))
                {
                    //時間測定開始
                    m_timer.Start();
                    if (CheckBoxSimulCapture.Checked == true)
                    {
                        //カメラ取り込み
                        m_video2.GrabImageASync(m_image2_CH0, m_image2_CH1);
                        m_video1.GrabImageSync(m_image1_CH0, m_image1_CH1);
                    }
                    else
                    {
                        //カメラ取り込み
                        m_video1.GrabImageASync(m_image1_CH0, m_image1_CH1);
                        m_video2.GrabImageASync(m_image2_CH0, m_image2_CH1);
                    }
                    while ((Status1 != 0) || (Status2 != 0))
                    {
                        Status1 = m_video1.GrabStatus;
                        Status2 = m_video2.GrabStatus;
                    }
                    //時間測定終了
                    msec = m_timer.Stop();
                    //測定時間表示
                    TextBoxTime.Text = msec.ToString("0.000");
                    //再描画
                    CFviImageViewVideo1CH0.Refresh();
                    CFviImageViewVideo1CH1.Refresh();
                    CFviImageViewVideo2CH0.Refresh();
                    CFviImageViewVideo2CH1.Refresh();
                }
                timer1.Enabled = true;
            }
            catch (FVIL.CFviException ex)
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video1 Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxVideo1Trigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo1Opened == true)
                {
                    if (CheckBoxSimulCapture.Checked == true)
                    {
                        m_video1.Trigger = (uint)ComboBoxVideo1Trigger.SelectedIndex + 1;
                    }
                    else
                    {
                        m_video1.Trigger = (uint)ComboBoxVideo1Trigger.SelectedIndex;
                    }
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video1={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video2 Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxVideo2Trigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo2Opened == true)
                {
                    if (CheckBoxSimulCapture.Checked == true)
                    {
                        m_video2.Trigger = (uint)ComboBoxVideo2Trigger.SelectedIndex + 1;
                    }
                    else
                    {
                        m_video2.Trigger = (uint)ComboBoxVideo2Trigger.SelectedIndex;
                    }
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video2={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Simultaneous Captureチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 同時取り込み設定状態の設定。
        /// </remarks>
        private void CheckBoxSimulCapture_CheckedChanged(object sender, EventArgs e)
        {
            int errflg = 0;
            try
            {
                if (m_flgVideo1Opened == true)
                {
                    errflg = 1;
                    if (CheckBoxSimulCapture.Checked == true)
                    {
                        //トリガモードコンボボックス初期化
                        ComboBoxVideo1Trigger.Items.Clear();
                        ComboBoxVideo1Trigger.Items.Add("内部トリガ");
                        ComboBoxVideo1Trigger.Items.Add("外部トリガ");
                        ComboBoxVideo1Trigger.SelectedIndex = 0;

                        //トリガ共有モードの設定
                        m_video1.TrgSharingMode = 2;      //マスターモード
                        //トリガモードの設定
                        m_video1.Trigger = 1;
                    }
                    else
                    {
                        //トリガモードコンボボックス初期化
                        ComboBoxVideo1Trigger.Items.Clear();
                        ComboBoxVideo1Trigger.Items.Add("ノーマル");
                        ComboBoxVideo1Trigger.Items.Add("内部トリガ");
                        ComboBoxVideo1Trigger.Items.Add("外部トリガ");
                        ComboBoxVideo1Trigger.SelectedIndex = 0;

                        //トリガ共有モードの設定
                        m_video1.TrgSharingMode = 0;      //独立モード
                        //トリガモードの設定
                        m_video1.Trigger = 0;
                    }
                }
                if (m_flgVideo2Opened == true)
                {
                    errflg = 2;
                    if (CheckBoxSimulCapture.Checked == true)
                    {
                        //トリガモードコンボボックス初期化
                        ComboBoxVideo2Trigger.Items.Clear();
                        ComboBoxVideo2Trigger.Items.Add("外部トリガ");
                        ComboBoxVideo2Trigger.SelectedIndex = 0;

                        //トリガ共有モードの設定
                        m_video2.TrgSharingMode = 1;      //スレーブモード
                        //トリガモードの設定
                        m_video2.Trigger = 2;
                    }
                    else
                    {
                        //トリガモードコンボボックス初期化
                        ComboBoxVideo2Trigger.Items.Clear();
                        ComboBoxVideo2Trigger.Items.Add("ノーマル");
                        ComboBoxVideo2Trigger.Items.Add("内部トリガ");
                        ComboBoxVideo2Trigger.Items.Add("外部トリガ");
                        ComboBoxVideo2Trigger.SelectedIndex = 0;

                        //トリガ共有モードの設定
                        m_video2.TrgSharingMode = 0;      //独立モード
                        //トリガモードの設定
                        m_video2.Trigger = 0;
                    }
                }
                //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                //キャプチャボードによってこの時間は異なります。
                System.Threading.Thread.Sleep(350);
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                switch (errflg)
                {
                    case 1:
                        MessageBox.Show(String.Format("ErrorCode:Video1={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        break;
                    case 2:
                        MessageBox.Show(String.Format("ErrorCode:Video2={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        break;
                }
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video1 Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void NumericUpDownVideo1Exposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo1Opened == true)
                {
                    m_video1.ExposureTime = (uint)NumericUpDownVideo1Exposure.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video2 Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void NumericUpDownVideo2Exposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo2Opened == true)
                {
                    m_video2.ExposureTime = (uint)NumericUpDownVideo2Exposure.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video2={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video1 Offset CH0値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// オフセット(CH0)の設定。
        /// </remarks>
        private void NumericUpDownVideo1CH0Offset_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo1Opened == true)
                {
                    //チャネル０に設定
                    m_video1.VideoOffsetCH0 = (int)NumericUpDownVideo1CH0Offset.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video1={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video1 Offset CH1値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// オフセット(CH1)の設定。
        /// </remarks>
        private void NumericUpDownVideo1CH1Offset_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo1Opened == true)
                {
                    //チャネル１に設定
                    m_video1.VideoOffsetCH1 = (int)NumericUpDownVideo1CH1Offset.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video1={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video2 Offset CH0値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// オフセット(CH0)の設定。
        /// </remarks>
        private void NumericUpDownVideo2CH0Offset_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo2Opened == true)
                {
                    //チャネル０に設定
                    m_video2.VideoOffsetCH0 = (int)NumericUpDownVideo2CH0Offset.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video2={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video2 Offset CH1値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// オフセット(CH1)の設定。
        /// </remarks>
        private void NumericUpDownVideo2CH1Offset_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo2Opened == true)
                {
                    //チャネル１に設定
                    m_video2.VideoOffsetCH1 = (int)NumericUpDownVideo2CH1Offset.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video2={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Binarize Mode設定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ２値化取込設定。
        /// </remarks>
        private void ComboBoxBinarize_SelectionChangeCommitted(object sender, EventArgs e)
        {
            int errflg = 0;
            try
            {
                m_imgtype = (GrabImagType)ComboBoxBinarize.SelectedIndex;

                if (m_flgVideo1Opened == true)
                {
                    errflg = 1;
                    FVIL.Video.CFviVideoBinarize bin = m_video1.Binarize;
                    bin.Mode = (uint)m_imgtype;
                    //２値化機能の設定
                    m_video1.BinarizeCH0 = bin;
                    m_video1.BinarizeCH1 = bin;
                    //取込画像を正規化
                    m_video1.ChangeImageSize(m_image1_CH0);
                    m_video1.ChangeImageSize(m_image1_CH1);
                }
                if (m_flgVideo2Opened == true)
                {
                    errflg = 2;
                    FVIL.Video.CFviVideoBinarize bin = m_video2.Binarize;
                    bin.Mode = (uint)m_imgtype;
                    //２値化機能の設定
                    m_video2.BinarizeCH0 = bin;
                    m_video2.BinarizeCH1 = bin;
                    //取込画像を正規化
                    m_video2.ChangeImageSize(m_image2_CH0);
                    m_video2.ChangeImageSize(m_image2_CH1);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                switch (errflg)
                {
                    case 1 :
                        MessageBox.Show(String.Format("ErrorCode:Video1={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        break;
                    case 2 :
                        MessageBox.Show(String.Format("ErrorCode:Video2={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        break;
                }
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video1 Threshold CH0値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 2値化しきい値(CH0)の設定。
        /// </remarks>
        private void NumericUpDownVideo1CH0Threshold_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo1Opened == true)
                {
                    FVIL.Video.CFviVideoBinarize bin = m_video1.Binarize;
                    bin.Threshold = (uint)NumericUpDownVideo1CH0Threshold.Value;
                    //チャネル０に設定
                    m_video1.BinarizeCH0 = bin;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video1={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video1 Threshold CH1値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ２値化しきい値(CH1)の設定。
        /// </remarks>
        private void NumericUpDownVideo1CH1Threshold_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo1Opened == true)
                {
                    FVIL.Video.CFviVideoBinarize bin = m_video1.Binarize;
                    bin.Threshold = (uint)NumericUpDownVideo1CH1Threshold.Value;
                    //チャネル１に設定
                    m_video1.BinarizeCH1 = bin;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video1={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video2 Threshold CH0値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 2値化しきい値(CH0)の設定。
        /// </remarks>
        private void NumericUpDownVideo2CH0Threshold_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo2Opened == true)
                {
                    FVIL.Video.CFviVideoBinarize bin = m_video2.Binarize;
                    bin.Threshold = (uint)NumericUpDownVideo2CH0Threshold.Value;
                    //チャネル０に設定
                    m_video2.BinarizeCH0 = bin;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video2={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video2 Threshold CH1値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ２値化しきい値(CH1)の設定。
        /// </remarks>
        private void NumericUpDownVideo2CH1Threshold_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideo2Opened == true)
                {
                    FVIL.Video.CFviVideoBinarize bin = m_video2.Binarize;
                    bin.Threshold = (uint)NumericUpDownVideo2CH1Threshold.Value;
                    //チャネル１に設定
                    m_video2.BinarizeCH1 = bin;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode:Video2={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// x1ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります

            //表示倍率の設定。
            CFviImageViewVideo1CH0.Display.Magnification = 1.0;
            CFviImageViewVideo1CH1.Display.Magnification = 1.0;
            CFviImageViewVideo2CH0.Display.Magnification = 1.0;
            CFviImageViewVideo2CH1.Display.Magnification = 1.0;
            //再描画
            CFviImageViewVideo1CH0.Refresh();
            CFviImageViewVideo1CH1.Refresh();
            CFviImageViewVideo2CH0.Refresh();
            CFviImageViewVideo2CH1.Refresh();
        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            CFviImageViewVideo1CH0.FitImageSize();
            CFviImageViewVideo1CH1.FitImageSize();
            CFviImageViewVideo2CH0.FitImageSize();
            CFviImageViewVideo2CH1.FitImageSize();
            //再描画
            CFviImageViewVideo1CH0.Refresh();
            CFviImageViewVideo1CH1.Refresh();
            CFviImageViewVideo2CH0.Refresh();
            CFviImageViewVideo2CH1.Refresh();
        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = CFviImageViewVideo1CH0.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            CFviImageViewVideo1CH0.Display.Magnification = Value;
            CFviImageViewVideo1CH1.Display.Magnification = Value;
            CFviImageViewVideo2CH0.Display.Magnification = Value;
            CFviImageViewVideo2CH1.Display.Magnification = Value;
            //再描画
            CFviImageViewVideo1CH0.Refresh();
            CFviImageViewVideo1CH1.Refresh();
            CFviImageViewVideo2CH0.Refresh();
            CFviImageViewVideo2CH1.Refresh();
        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = CFviImageViewVideo1CH0.Display.Magnification;
            Value = Value - 0.5;
            if ((Value <= 0))
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            CFviImageViewVideo1CH0.Display.Magnification = Value;
            CFviImageViewVideo1CH1.Display.Magnification = Value;
            CFviImageViewVideo2CH0.Display.Magnification = Value;
            CFviImageViewVideo2CH1.Display.Magnification = Value;
            //再描画
            CFviImageViewVideo1CH0.Refresh();
            CFviImageViewVideo1CH1.Refresh();
            CFviImageViewVideo2CH0.Refresh();
            CFviImageViewVideo2CH1.Refresh();
        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                CFviImageViewVideo1CH0.Display.OverlayAscope.Enable = true;
                CFviImageViewVideo1CH0.Display.OverlayAscope.Active = true;
                CFviImageViewVideo1CH1.Display.OverlayAscope.Enable = true;
                CFviImageViewVideo1CH1.Display.OverlayAscope.Active = true;
                CFviImageViewVideo2CH0.Display.OverlayAscope.Enable = true;
                CFviImageViewVideo2CH0.Display.OverlayAscope.Active = true;
                CFviImageViewVideo2CH1.Display.OverlayAscope.Enable = true;
                CFviImageViewVideo2CH1.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                CFviImageViewVideo1CH0.Display.OverlayAscope.Enable = false;
                CFviImageViewVideo1CH0.Display.OverlayAscope.Active = false;
                CFviImageViewVideo1CH1.Display.OverlayAscope.Enable = false;
                CFviImageViewVideo1CH1.Display.OverlayAscope.Active = false;
                CFviImageViewVideo2CH0.Display.OverlayAscope.Enable = false;
                CFviImageViewVideo2CH0.Display.OverlayAscope.Active = false;
                CFviImageViewVideo2CH1.Display.OverlayAscope.Enable = false;
                CFviImageViewVideo2CH1.Display.OverlayAscope.Active = false;
            }
            //再描画
            CFviImageViewVideo1CH0.Refresh();
            CFviImageViewVideo1CH1.Refresh();
            CFviImageViewVideo2CH0.Refresh();
            CFviImageViewVideo2CH1.Refresh();
        }

        /// <summary>
        /// ReLoad Ini Fileボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ビデオ設定用iniファイルの再設定。
        /// </remarks>
        private void ButtonReLoadIni_Click(object sender, EventArgs e)
        {
            //フリーラン中ならば、停止する。
	        if (m_flgFreerun == true) {
		        ButtonGrabContinue.Text = "Grab Continue";
		        ButtonGrabOne.Enabled = true;
		        m_flgFreerun = false;
		        timer1.Enabled = false;
		        Application.DoEvents();
	        }

        	//iniファイル選択
	        fnSetIniFile();
        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    String name;
                    char[] ex;
                    int temp;
                    name = ".";
                    ex = name.ToCharArray();
                    //拡張子までの文字数を算出
                    temp = ofdlg.FileName.Length - ofdlg.FileName.LastIndexOfAny(ex);
                    temp = ofdlg.FileName.Length - temp;

                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 

                    //ビデオ１チャネル０の画像保存
                    name = ofdlg.FileName.Insert(temp, "_Video1CH0");
                    FVIL.File.Function.SaveImageFile(name, CFviImageViewVideo1CH0.Image);
                    //ビデオ１チャネル１の画像保存
                    name = ofdlg.FileName.Insert(temp, "_Video1CH1");
                    FVIL.File.Function.SaveImageFile(name, CFviImageViewVideo1CH1.Image);
                    //ビデオ２チャネル０の画像保存
                    name = ofdlg.FileName.Insert(temp, "_Video2CH0");
                    FVIL.File.Function.SaveImageFile(name, CFviImageViewVideo2CH0.Image);
                    //ビデオ２チャネル１の画像保存
                    name = ofdlg.FileName.Insert(temp, "_Video2CH1");
                    FVIL.File.Function.SaveImageFile(name, CFviImageViewVideo2CH1.Image);

                    //保存するファイル形式が固定の場合は、以下のように記述します。
                    //例1) bmpファイルの場合
                    //Dim file As FVIL.File.CFviImageFileBmp = New FVIL.File.CFviImageFileBmp()
                    //file.Save(ofdlg.FileName, m_image)

                    //例2) jpgファイルの場合
                    //Dim file As FVIL.File.CFviImageFileJpg = New FVIL.File.CFviImageFileJpg()
                    //file.Save(ofdlg.FileName, m_image)
                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch (ex.ErrorCode)
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FVC05_4CHSAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            // ビデオデバイスがオープンされていた場合はクローズする
            if (m_flgVideo1Opened)
            {
                m_video1.Close();
                m_flgVideo1Opened = false;
            }
            if (m_flgVideo2Opened)
            {
                m_video2.Close();
                m_flgVideo2Opened = false;
            }
        }

    }
}